import numpy as np
from scipy.fftpack import dct

DIC_GENERATIONS = ['gaussian', 'uniform', 'dct', 'toeplitz']
OBS_GENERATIONS = ['gaussian', 'posgaussian']

def _normalize(x):
    if len(x.shape) == 2:
        for i in range(x.shape[1]):
            x[:, i] /= np.linalg.norm(x[:, i], 2)
    elif len(x.shape) == 1:
        x /= np.linalg.norm(x,2)
    else:
        raise ValueError("Require x with 1 or 2 dimensions.")
    return x

def _gaussian_dic(m, n):
    A = np.random.randn(n, m).T
    return A

def _uniform_dic(m, n):
    A = np.random.rand(n, m).T
    return A

def _dct_dic(m, n):
    A = dct(np.eye(max(m,n)))
    indices = np.random.permutation(n)
    A = A[indices,:]
    A = A[:n,:m].T
    return A

def _toeplitz_dic(m, n):
    ranget = np.linspace(-10, 10, m)
    offset = 3.
    rangemu = np.linspace(np.min(ranget)+offset, np.max(ranget)-offset, n)
    A = np.zeros((m, n))
    for j in range(n):
        A[:,j] = np.exp(-.5 * (ranget - rangemu[j])**2)
    return A

def _gaussian_obs(m,pos=False):
    y = np.random.randn(m)
    if pos:
        y = np.maximum(y,0)
    return y

def sample_data(m,n,dic_gen,obs_gen):
    """
    Sample a normalized (m,n) matrix and a normalized (m,) vector using given 
    generation methods.

    Parameters
    ----------
    m: int
        Dimension of y.
    n: int
        Number of columns in A.
    dic_gen: {'gaussian', 'uniform', 'dct', 'toeplitz'}
        Generation method for A.
    dic_gen: {'gaussian', 'posgaussian'}
        Generation method for y.
    """

    if dic_gen not in DIC_GENERATIONS:
        raise ValueError(f"Values allowed for parameter dic_gen are " \
                          " {DIC_GENERATIONS}.")
    if obs_gen not in OBS_GENERATIONS:
        raise ValueError(f"Values allowed for parameter obs_gen are " \
                          " {OBS_GENERATIONS}.")

    if dic_gen == 'gaussian':
        A = _gaussian_dic(m, n)
    elif dic_gen == 'uniform':
        A = _uniform_dic(m, n)
    elif dic_gen == 'dct':
        A = _dct_dic(m, n)
    elif dic_gen == 'toeplitz':
        A = _toeplitz_dic(m, n)

    if obs_gen == 'gaussian':
        y = _gaussian_obs(m,False)
    elif obs_gen == 'posgaussian':
        y = _gaussian_obs(m,True)

    A = _normalize(A)
    y = _normalize(y)

    return A, y
